<order>
    <style>
        #order-search-items {
            margin-top: 32px;
            margin-bottom: 12px;
            margin-right: 25px;
        }
        #order-search-items label {
            margin-left: 10px;
            margin-right: 10px;
            margin-top: 5px;
        }
        .insertSpace {
            width: 390px;
        }
        .search-item {
            display: flex;
        }
        .search-item #orderNo {
            width: 200px;
        }
        #from, #to {
            width: 120px;
        }
    </style>
    <div id="order-search-items" class="hide">
        <div class="search-item">
            <label for="from">From: </label>
            <input type="text" id="from" name="from" class="datetimepicker form-control" value="{ start}"/>
            <label for="to">To: </label>
            <input type="text" id="to" name="to" class="datetimepicker form-control" value="{ end }"/>
            <label for="orderNo">注文番号: </label>
            <input type="text" id="orderNo" name="orderNo" class="form-control"/>
        </div>
    </div>
    <data/>
    <script>
        export default {
            onBeforeMount(props, state) {
                this.scrutiny = props.scrutiny
                this.logoutModal = props.logoutModal
                this.start = moment().format('YYYY-MM-DD')
                this.end = moment().format('YYYY-MM-DD')
            },
            onMounted(props, states) {
                $('.datetimepicker').datetimepicker({
                    dayViewHeaderFormat: 'YYYY年MM月',
                    stepping: 1,
                    locale:'ja',
                    format : 'YYYY-MM-DD'
                });
                clearErrorMessage()
                RiotUtil.next("data", {
                    scrutiny: this.scrutiny,
                    logoutModal: this.logoutModal,
                    csvImport: true,
                    title: "注文管理",
                    name: 'order',
                    order: [[6, 'desc'], [3, 'asc']],
                    items: ['取込日', "購入者", "注文番号", '端末番号', "注文日", "プラン名", "お届け先名", "電話番号", "郵便番号", "住所", "備考"],
                    service: new OrderService(),
                    assetService: new AssetService(),
                    codeService: new CodeService(),
                    disabledColumns: [2,3,4,6,7,8,9,10,11],
                    fixedColumns: {leftColumns: 4},
                    leftButtons: [{
                        label: '検索',
                        onclick: function (e) {
                            $("#order_datatables").dataTable().api().ajax.reload()
                        }
                    }],
                    ajax: async function (handler) {

                        const orderNo = $("#orderNo").val()
                        let response
                        if (orderNo) {
                            response = await this.service.get(orderNo)
                            if (response.status != "OK" || !response.data) {
                                return handler([])
                            }
                            response.data = [response.data]
                        } else {
                            const email = ''
                            let start = $("#from").val()
                            let end = $("#to").val()
                            response = await this.service.search(email, start, end)
                            if (response.status != "OK" || !response.data || response.data.length < 1) {
                                return handler([])
                            }
                        }

                        const jsons = []
                        response.data.forEach(data => {
                            const json = {
                                importDate: data.importDate,
                                orderNo: data.orderNo,
                                assetNo: data.assetNo ? data.assetNo : '',
                                orderDate: data.orderDate,
                                buyerName: data.buyerName,
                                planName: data.planName,
                                shipName: data.shipName,
                                shipPhone: data.shipPhone,
                                shipPostCode: data.shipPostCode,
                                shipAddress: data.shipAddress,
                                memo: data.memo ? data.memo : '',
                                rowId: data.rowId ? data.rowId : Math.random()
                            }
                            // 注文日は1週間以内
                            // const orderDate = moment(json.orderDate)
                            // const diff = moment().diff(orderDate, 'days')
                            // if (diff < 8) {
                                jsons.push(json)
                            // }
                        })
                        handler(jsons)
                    },
                    csvUpload: async function(e, dataTable, editor, conf, files, generalError, completeCallback, ids) {

                        const fileName = files[0].name

                        const base64 = e.target.result
                        const csv = Encoding.convert(new Uint8Array(base64), {to: 'unicode', from: 'SJIS', type:'string'})
                        const data = csv.replace(/\n/g, '\t').split('\t').filter(function(d,i) {
                            return i > 29
                        })

                        const rows = []
                        let row = {
                            email: UserInfo.getInstance().getEmail(),
                            importDate: moment().format('YYYY-MM-DD')
                        }
                        for (let i = 0; i < data.length; i++) {
                            switch (i % 30) {
                                case 0:
                                    row.orderNo = data[i]
                                    break
                                case 3:
                                    row.orderDate = moment(data[i]).format("YYYY-MM-DD")
                                    break
                                case 5:
                                    row.buyerName = data[i]
                                    break
                                case 8:
                                    const start = data[i].indexOf('【') + 1
                                    const end = data[i].indexOf('】')
                                    row.planName = data[i].slice(start, end)
                                    break
                                case 9:
                                    row.quantity = data[i]
                                    break
                                case 16:
                                    row.shipName = data[i]
                                    break
                                case 17:
                                    row.shipAddress =  data[i+4] + data[i] + data[i+1] + data[i+2]
                                    break
                                case 22:
                                    row.shipPostCode = data[i]
                                    break
                                case 24:
                                    row.shipPhone = data[i]
                                    break
                                case 29:
                                    rows.push(row)
                                    if (row.quantity > 1) {
                                        for (let j = 1; j < row.quantity; j++) {
                                            const clone = Object.assign({}, row)
                                            clone.orderNo = `${clone.orderNo}-${j+1}`
                                            rows.push(clone)
                                        }
                                    }

                                    row = {
                                        email: UserInfo.getInstance().getEmail(),
                                        importDate: moment().format('YYYY-MM-DD')
                                    }
                            }
                        }

                        const response = await fetch(conf.ajax, {
                            method: 'POST',
                            body: JSON.stringify({
                                email: UserInfo.getInstance().getEmail(),
                                rows,
                                rawData: csv,
                                fileName
                            })
                        })

                        if (response.ok) {
                            const json = await response.json()
                            editor.off(`preSubmit.DTE_Upload`);
                            editor._event('uploadXhrSuccess', [conf.name, json]);
                            toastr.success(json.data.successCount + '件取込完了しました。')

                            if (json.fieldErrors && json.fieldErrors.length) {
                                const errors = json.fieldErrors;

                                for (let i = 0, ien = errors.length; i < ien; i++) {
                                    editor.error(errors[i].name, errors[i].status);
                                }
                            } else if (json.error) {
                                editor.error(json.error);
                            } else {
                                completeCallback.call(editor, ids);
                                editor.submit();

                                dataTable.ajax.reload()
                                conf._input.find('div.upload button').html(conf.uploadText)
                            }
                        } else {
                            const xhr = await response.json()
                            editor._event('uploadXhrError', [conf.name, xhr]);
                            editor.error(conf.name, generalError);
                            toastr.error('取込エラーが発生しました。')
                        }
                    },
                    initComplete: function(dt, data) {

                        $('#order-search-items').removeClass('hide')
                        $('admincontent #search-menu').prepend($('#order-search-items'))
                    },
                    draw: function() {
                    },
                    preSubmit: function (e, json, action, self) {

                        if (action == 'remove') return

                        const orderNo = self.field('orderNo')
                        if (!v8n().not.null().not.empty().test(orderNo.get())) {
                            orderNo.error('注文番号は必須です')
                        }

                        const target = json.data[_.keys(json.data)[0]]
                        if (action == 'create') {
                        }

                        target.email = UserInfo.getInstance().getEmail()

                        if (self.inError()) {
                            return false
                        }
                    },
                    submitSuccess: async function (e, json, data) {
                        const api = $('#order_datatables').dataTable().api()

                        if (!data || !data.assetNo) return
                        const assetNo = data.assetNo

                        const index = api.data().toArray().findIndex(u => {return u.orderNo == data.orderNo})

                        const response = await this.assetService.get(assetNo)
                        if (response.statusCode != 200 || !response.data || !response.data.length) {
                            toastr.error('指定された端末は存在しません。')
                            const order = api.data().toArray()[index]
                            order.assetNo = ''
                            await this.service.update(order)
                            api.ajax.reload()
                            return
                        }

                        const asset = response.data[0]
                        const params = Object.assign({}, asset)
                        params.shipName = data.buyerName == data.shipName ? data.buyerName : `${data.buyerName}(${data.shipName})`
                        params.start = moment().format('YYYY-MM-DD')
                        params.contactTel = data.shipPhone
                        params.contactAddress = data.shipAddress
                        params.orderNo = data.orderNo
                        params.kindType = data.planName
                        params.memo = data.memo

                        if (!params.status) {
                            params.status = 1
                        }

                        const planName = data.planName
                        const codeResp = await this.codeService.get('plan')
                        if (codeResp.statusCode != 200) {
                            toastr.error(codeResp.message)
                            return
                        }
                        const code = codeResp.data.find(function (code) {
                            return code.key == planName
                        })
                        if (code) {
                            params.end = moment().add(code.value, 'days').format('YYYY-MM-DD')
                        }

                        // params.updatedAt = moment().format('YYYY-MM-DD HH:mm:ss')
                        if (asset.userName || asset.start || asset.end  || asset.shipAddress
                            || asset.contactTel || asset.contactAddress || asset.memo || asset.orderNo ||asset.kindType) {
                            // ワーニング
                            riot.unmount('commonmodal', {}, 'submitordermodal')
                            const orderModal = riot.mount('commonmodal',{
                                data: params,
                                api,
                                orderNo: params.orderNo
                            }, 'submitordermodal')[0]
                            orderModal.show()
                            return
                        }

                        await this.assetService.update(params)
                    },
                    addJSON: async function (json) {
                        return json
                    },
                    open: function (e, json, action, editor) {

                        if (action == 'edit') {
                            editor.field('orderNo').disable()
                        } else {
                            editor.field('orderNo').enable()
                        }

                        if (action == 'create') {
                        }
                    },
                    buttonsAction: function ( e, buttonApi, dataTable, node, config ) {
                        const length = $('.drop').length
                        if (length < 1) return
                        const row = $('.row.second')
                        $('form').after(row)
                    },
                    actionButtons: [{
                        text: "ダウンロード",
                        func: async function (e, dt, editor, self) {
                            const email = ''
                            const path = UserInfo.getInstance().getEmail()
                            let start = $("#from").val()
                            let end = $("#to").val()
                            const service = new OrderService()
                            const response = await service.downloadCsv(email, start, end, path, 'false')
                            if (response.statusCode != 200) {
                                toastr.error(response.message)
                                return
                            }
                            const s3Path = response.data
                            riot.unmount('commonmodal', {}, 'assetmodal')
                            const s3Modal = riot.mount('commonmodal',{
                                url: s3Path,
                            }, 'assets3modal')[0]
                            s3Modal.show()
                        }
                    },{
                        text: "自社取込",
                        func: async function (e, dt, editor, self) {
                            riot.unmount('commonmodal', {}, 'assetmodal')
                            const captureOrderCsvModal = riot.mount('commonmodal',{
                                service: new OrderService()
                            }, 'captureordercsv')[0]
                            captureOrderCsvModal.show()
                        }
                    },{
                        text: "Yes取込",
                        func: async function (e, dt, editor, self) {
                            riot.unmount('commonmodal', {}, 'assetmodal')
                            const captureOrderCsvModal = riot.mount('commonmodal',{
                                service: new OrderService()
                            }, 'captureorderyescsv')[0]
                            captureOrderCsvModal.show()
                        }
                    },{
                        text: "出荷CSV",
                        func: async function (e, dt, editor, self) {
                            const email = ''
                            const path = UserInfo.getInstance().getEmail()
                            let start = $("#from").val()
                            let end = $("#to").val()
                            const service = new OrderService()
                            const response = await service.downloadCsv(email, start, end, path, 'true')
                            if (response.statusCode != 200) {
                                toastr.error(response.message)
                                return
                            }
                            const s3Path = response.data
                            riot.unmount('commonmodal', {}, 'assetmodal')
                            const s3Modal = riot.mount('commonmodal', {
                                url: s3Path,
                            }, 'assets3modal')[0]
                            s3Modal.show()
                        }
                    }]
                }, "datatables")
            }
        }

        function clearErrorMessage() {
            $("#error-message").text('').addClass('hide')
        }
        function setErrorMessage(message) {
            $("#error-message").text(message).removeClass('hide')
        }
    </script>
</order>
